package com.cpp.web.controller.largeScreen;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cpp.common.core.domain.R;
import com.cpp.system.service.ISysConfigService;
import com.cpp.web.domain.station.*;
import com.cpp.web.service.station.*;
import com.cpp.web.utils.DateTimeUtil;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;


/**
 * 大屏模块
 *
 * @author whc
 * @date 2022-03-18 15:49:14
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/largeScreenController")
public class LargeScreenController {
    @Autowired
    PowerStationStatusDataService powerStationStatusDataService;
    @Autowired
    ISysConfigService configService;
    @Autowired
    ForecastPowerUltraShortTermStationService forecastPowerUltraShortTermStationService;
    @Autowired
    ForecastPowerShortTermRegulationService forecastPowerShortTermRegulationService;
    @Autowired
    ElectricFieldService electricFieldService;
    @Autowired
    WindTowerStatusDataService windTowerStatusDataService;
    @Autowired
    WeatherStationStatusDataService weatherStationStatusDataService;

    @GetMapping("/getBizData")
    public R getBizData() {
        Date dayStartTime = DateTimeUtil.getDayStartTime(System.currentTimeMillis());
        Date dayEndTime = DateTimeUtil.getDayLastTime(System.currentTimeMillis());

        Map bizDataMap = new HashMap();
        // 定义首页实际功率、可用、理论总和
        BigDecimal realSum = BigDecimal.ZERO;
        BigDecimal ableSum = BigDecimal.ZERO;
        BigDecimal theorySum = BigDecimal.ZERO;
        BigDecimal cdqSum = BigDecimal.ZERO;
        BigDecimal dqSum = BigDecimal.ZERO;
        // 首页实际、可用、理论曲线图数组
        List<Object> realList = new ArrayList<>();
        List<Object> ableList = new ArrayList<>();
        List<Object> theoryList = new ArrayList<>();
        List<Object> cdqList = new ArrayList<>();
        List<Object> dqList = new ArrayList<>();
        // 限电场站数定义
        int xdCount = 0;
        // 查询实际功率、可用、理论
        Date currentTime5 = DateTimeUtil.getMomentTimeFor5Minute(System.currentTimeMillis());
        List<PowerStationStatusData> powerStationStatusDataList = powerStationStatusDataService.findByTimeBetween(dayStartTime,dayEndTime);
        // 按场时间分组
        Map<Long, List<PowerStationStatusData>> powerStationStatusDataGroup = powerStationStatusDataList.stream().collect(Collectors.groupingBy(s->s.getTime().getTime()));
        // 统计当前时刻实际功率的总和
        if (powerStationStatusDataGroup.get(currentTime5.getTime())!=null){
            List<PowerStationStatusData> currentPowerStationDataList = powerStationStatusDataGroup.get(currentTime5.getTime());
            for (PowerStationStatusData powerStationStatusData:currentPowerStationDataList){
                realSum = realSum.add(powerStationStatusData.getRealValue());
                ableSum = ableSum.add(powerStationStatusData.getAbleValue());
                theorySum = theorySum.add(powerStationStatusData.getTheoryValue());
                if (powerStationStatusData.getPowerRationing().intValue()==1){
                    xdCount++;
                }
            }
        }


        // 获取超短期
        String cdqHowLongAgo = configService.selectConfigByKey("cdqHowLongAgo");
        QueryWrapper cdqysWrapper = new QueryWrapper<>();
        cdqysWrapper.eq("forecast_how_long_ago", Integer.parseInt(cdqHowLongAgo));
        cdqysWrapper.between("time", dayStartTime, dayEndTime);
        List<ForecastPowerUltraShortTermStation> forecastPowerUltraShortTermStationList = forecastPowerUltraShortTermStationService.list(cdqysWrapper);
        // 按场时间分组
        Map<Long, List<ForecastPowerUltraShortTermStation>> utraShortTermStationDataGroup = forecastPowerUltraShortTermStationList.stream().collect(Collectors.groupingBy(s->s.getTime().getTime()));
        Date currentTime15 = DateTimeUtil.getMomentTimeFor15Minute(System.currentTimeMillis());
        // 统计当前时刻超短期的总和
        if (utraShortTermStationDataGroup.get(currentTime15.getTime())!=null){
            List<ForecastPowerUltraShortTermStation> ultraShortTermDataList = utraShortTermStationDataGroup.get(currentTime15.getTime());
            for (ForecastPowerUltraShortTermStation forecastPowerUltraShortTermStation:ultraShortTermDataList){
                cdqSum = cdqSum.add(forecastPowerUltraShortTermStation.getFpValue());
            }
        }

        // 获取短期上报
        String dqHowLongAgo = configService.selectConfigByKey("dqHowLongAgo");
        QueryWrapper dqsbWrapper = new QueryWrapper<>();
        dqsbWrapper.eq("forecast_how_long_ago", Integer.parseInt(dqHowLongAgo));
        dqsbWrapper.between("time", dayStartTime, dayEndTime);
        List<ForecastPowerShortTermRegulation> forecastPowerShortTermRegulationList = forecastPowerShortTermRegulationService.list(dqsbWrapper);
        // 按场时间分组
        Map<Long, List<ForecastPowerShortTermRegulation>> shortTermDataGroup = forecastPowerShortTermRegulationList.stream().collect(Collectors.groupingBy(s->s.getTime().getTime()));
        // 统计当前时刻短期的总和
        if (shortTermDataGroup.get(currentTime15.getTime())!=null){
            List<ForecastPowerShortTermRegulation> shortTermDataList = shortTermDataGroup.get(currentTime15.getTime());
            for (ForecastPowerShortTermRegulation forecastPowerShortTermRegulation:shortTermDataList){
                dqSum = dqSum.add(forecastPowerShortTermRegulation.getFpValue());
            }
        }

        Long momentTime = 15 * 60 * 1000L;
        for (Long tempTime = dayStartTime.getTime(); tempTime <= dayEndTime.getTime(); tempTime = tempTime + momentTime) {
            // 统计实际功率、可用、理论曲线
            if (powerStationStatusDataGroup.get(tempTime)!=null){
                // 对每个时间点的所有场站数据进行求和
                List<PowerStationStatusData> everyDataList = powerStationStatusDataGroup.get(tempTime);
                BigDecimal tempRealSum = BigDecimal.ZERO;
                BigDecimal tempAbleSum = BigDecimal.ZERO;
                BigDecimal tempTheorySum = BigDecimal.ZERO;
                for (PowerStationStatusData powerStationStatusData:everyDataList){
                    BigDecimal realValue = powerStationStatusData.getRealValue();
                    tempRealSum = tempRealSum.add(realValue);
                    BigDecimal ableValue = powerStationStatusData.getAbleValue();
                    tempAbleSum = tempAbleSum.add(ableValue);
                    BigDecimal theoryValue = powerStationStatusData.getTheoryValue();
                    tempTheorySum = tempTheorySum.add(theoryValue);
                }
                realList.add(tempRealSum);
                ableList.add(tempAbleSum);
                theoryList.add(tempTheorySum);
            }
            else{
                realList.add(null);
                ableList.add(null);
                theoryList.add(null);
            }
            // 统计超短期曲线
            if (utraShortTermStationDataGroup.get(tempTime)!=null){
                // 对每个时间点的所有场站数据进行求和
                List<ForecastPowerUltraShortTermStation> everyDataList = utraShortTermStationDataGroup.get(tempTime);
                BigDecimal tempCdqSum = BigDecimal.ZERO;
                for (ForecastPowerUltraShortTermStation forecastPowerUltraShortTermStation:everyDataList){
                    BigDecimal cdqValue = forecastPowerUltraShortTermStation.getFpValue();
                    tempCdqSum = tempCdqSum.add(cdqValue);
                }
                cdqList.add(tempCdqSum);
            }
            else{
                cdqList.add(null);
            }
            // 统计短期上报曲线
            if (shortTermDataGroup.get(tempTime)!=null){
                // 对每个时间点的所有场站数据进行求和
                List<ForecastPowerShortTermRegulation> everyDataList = shortTermDataGroup.get(tempTime);
                BigDecimal tempDqSum = BigDecimal.ZERO;
                for (ForecastPowerShortTermRegulation forecastPowerShortTermRegulation:everyDataList){
                    BigDecimal dqValue = forecastPowerShortTermRegulation.getFpValue();
                    tempDqSum = tempDqSum.add(dqValue);
                }
                dqList.add(tempDqSum);
            }
            else{
                dqList.add(null);
            }
        }


        // 定义总和map
        Map sumMap = new HashMap();
        sumMap.put("realSum",realSum);
        sumMap.put("ableSum",ableSum);
        sumMap.put("theorySum",theorySum);
        // 定义曲线map
        Map curveMap = new HashMap();
        curveMap.put("realList",realList);
        curveMap.put("ableList",ableList);
        curveMap.put("theoryList",theoryList);
        sumMap.put("cdqSum",cdqSum);
        curveMap.put("cdqList",cdqList);
        sumMap.put("dqSum",dqSum);
        curveMap.put("dqList",dqList);
        // 获取所有场站
        List<ElectricField> electricFieldList = electricFieldService.list();
        // 定义限电图map
        Map xdMap = new HashMap();
        // 总场站数
        xdMap.put("zczs",electricFieldList.size());
        // 限电场站数
        xdMap.put("xdczs",xdCount);
        // 限电比
        if (electricFieldList.size()==0){
            xdMap.put("xdbl","0");
        }
        else{
            xdMap.put("xdbl",new BigDecimal(xdCount).divide(new BigDecimal(electricFieldList.size()).multiply(new BigDecimal("100")),2, BigDecimal.ROUND_HALF_UP));
        }
        // 总场站容量
        BigDecimal totalCapacity = BigDecimal.ZERO;
        for (ElectricField electricField : electricFieldList) {
            totalCapacity = totalCapacity.add(electricField.getCapacity());
        }
        xdMap.put("totalCapacity",totalCapacity);

        // 首页风速、辐照度平均值统计
        Map envDataMap = new HashMap();
        BigDecimal wsAvg = BigDecimal.ZERO;
        // 获取当前时刻的所有风塔数据
        List<WindTowerStatusData> windTowerStatusDataList = windTowerStatusDataService.findByBetweenTimeAndStationCode(currentTime5, currentTime5, null);
        for (WindTowerStatusData windTowerStatusData:windTowerStatusDataList){
            wsAvg = wsAvg.add(windTowerStatusData.getWsHubHeight());
        }
        if (wsAvg.compareTo(BigDecimal.ZERO)==1){
            wsAvg = wsAvg.divide(new BigDecimal(windTowerStatusDataList.size()),2, BigDecimal.ROUND_HALF_UP);
        }
        envDataMap.put("wsAvg",wsAvg);
        BigDecimal fzdAvg = BigDecimal.ZERO;
        // 获取当前时刻的所有气象站数据
        List<WeatherStationStatusData> weatherStationStatusDataList = weatherStationStatusDataService.findByBetweenTimeAndStationCode(currentTime5, currentTime5, null);
        for (WeatherStationStatusData weatherStationStatusData:weatherStationStatusDataList){
            fzdAvg = fzdAvg.add(weatherStationStatusData.getGlobalR());
        }
        if (fzdAvg.compareTo(BigDecimal.ZERO)==1){
            fzdAvg = fzdAvg.divide(new BigDecimal(weatherStationStatusDataList.size()),2, BigDecimal.ROUND_HALF_UP);
        }
        envDataMap.put("fzdAvg",fzdAvg);

        bizDataMap.put("sumMap",sumMap);
        bizDataMap.put("curveMap",curveMap);
        bizDataMap.put("xdMap",xdMap);
        bizDataMap.put("envDataMap",envDataMap);
        return R.ok(bizDataMap);
    }
}
