package com.cpp.web.service.overhaulplan.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cpp.web.domain.BaseForecastEntity;
import com.cpp.web.domain.datafactory.enums.FileTypeEnum;
import com.cpp.web.domain.overhaulplan.OverhaulPlan;
import com.cpp.web.domain.overhaulplan.OverhaulPlanRecords;
import com.cpp.web.domain.station.ElectricField;
import com.cpp.web.mapper.overhaulplan.OverhaulPlanMapper;
import com.cpp.web.service.overhaulplan.OverhaulPlanRecordsService;
import com.cpp.web.service.overhaulplan.OverhaulPlanService;
import com.cpp.web.service.station.ElectricFieldService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * cpp_overhaul_plan
 *
 * @author fxy
 * @date 2024-10-08 9:47:33
 */
@Service
@RequiredArgsConstructor
public class OverhaulPlanServiceImpl extends ServiceImpl<OverhaulPlanMapper, OverhaulPlan> implements OverhaulPlanService {

    private final OverhaulPlanMapper overhaulPlanMapper;

    private final ElectricFieldService electricFieldService;

    private final OverhaulPlanRecordsService overhaulPlanRecordsService;

    /**
     * 查询所有检修计划
     *
     * @return
     */
    @Override
    public QueryWrapper<OverhaulPlan> getList() {
        QueryWrapper<OverhaulPlan> wrapper = new QueryWrapper<>();
        return wrapper;
    }

    /**
     * 根据场站编号查询检修计划信息
     *
     * @param stationCode
     * @return
     */
    @Override
    public QueryWrapper<OverhaulPlan> getByStationCode(String stationCode, Integer status) {
        QueryWrapper<OverhaulPlan> wrapper = new QueryWrapper<>();
        if (null != status) {
            if (!"".equals(stationCode)) {
                wrapper.eq("station_code", stationCode);
                wrapper.eq("status", status);
            }
        } else {
            if (!"".equals(stationCode)) {
                wrapper.eq("station_code", stationCode);
            }
        }
        return wrapper;
    }

    @Override
    public List<OverhaulPlan> findByStationCode(String stationCode) {
        QueryWrapper<OverhaulPlan> wrapper = new QueryWrapper<>();
        if (null != stationCode && !"".equals(stationCode)){
            wrapper.eq("station_code",stationCode);
        }
        wrapper.eq("status",1);
        return list(wrapper);
    }

    @Override
    public List<OverhaulPlan> findByStationCodeAndTimeBetween(Date date, String stationCode) {
        QueryWrapper<OverhaulPlan> wrapper = new QueryWrapper<>();
        if (null != date) {
            wrapper.ge("start_time", date);
            wrapper.le("end_time", date);
        }
        if (null != stationCode && !"".equals(stationCode)) {
            wrapper.eq("station_code", stationCode);
        }
        wrapper.eq("status", 1);
        return overhaulPlanMapper.selectList(wrapper);
    }

    /**
     * 返回修正后数据
     *
     * @param list
     * @param <T>
     * @return
     */
    @Override
    public  List<BaseForecastEntity> getCorrectionData(List<BaseForecastEntity> list, String fileType, String fileName, String stationCode) {


        return list;
    }
}
